function [ k, l ] = register_gridimages( method, data, p, imgrid1, imgrid2, PHI1_X, PHI1_Y, PHI2_X, PHI2_Y )
% compensate the uniform translation of 2 pi x k from PHI1_Y to PHI2_Y
% and of 2 pi x l between PHI1_X to PHI2_X
% 1) input: 
% method: 1 = hand-picked points, 2 =  by cross-correlation on the image grids
% data: area on which the NCC is performed (not containing a hole or a
% crack) in method 1, or coordinates of the two corresponding points in
% method 2 
% p: pattern pitch
% imgrid1, imgrid2: grid images
% PHI1_X, PHI1_Y, PHI2_X, PHI2_Y: unwrapped phase maps
% 2) output:
% k,l

if (method==1)
   k = -round( ( (data(4)+PHI2_Y(data(4),data(3))*p/(2*pi)) - (data(2)+PHI1_Y(data(2),data(1))*p/(2*pi)) )/p );
   l = -round( ( (data(3)+PHI2_X(data(4),data(3))*p/(2*pi)) - (data(1)+PHI1_X(data(2),data(1))*p/(2*pi)) )/p );
end
    
if (method==2)

    [X,Y]=meshgrid(1:size(imgrid1,2),1:size(imgrid1,1));
    interpimgrid1 = interp2(X,Y,imgrid1,X-PHI1_X*p/(2*pi),Y-PHI1_Y*p/(2*pi),'cubic');
    interpimgrid2 = interp2(X,Y,imgrid2,X-PHI2_X*p/(2*pi),Y-PHI2_Y*p/(2*pi),'cubic');

    % maximum value for the range search of the matching window 
    % (try a larger value if the true motion is out of reach)
    supk=10;
    supl=10;
    
    % range search for k*p and l*p
    k_range=(-supk:1:supk)*p;
    l_range=(-supl:1:supl)*p;

    interpimgrid1=interpimgrid1(data(1):data(2),data(3):data(4));
    interpimgrid2=interpimgrid2(data(1):data(2),data(3):data(4));
    area_interpimgrid1=interpimgrid1(k_range(end)+1:end-k_range(end)-1,l_range(end)+1:end-l_range(end)-1);

    % search for a matching window maximizing the normalized cross-correlation (NCC)
    mean1=mean(area_interpimgrid1(:));
    std1=std(area_interpimgrid1(:));
    maxNCC=-1;
    for k=k_range,
        for l=l_range,        
            area_interpimgrid2=interpimgrid2(k_range(end)-k+1:end-k_range(end)-k-1,l_range(end)-l+1:end-l_range(end)-l-1);
            mean2=mean(area_interpimgrid2(:));  
            std2=std(area_interpimgrid2(:));
            curNCC=mean((area_interpimgrid1(:)-mean1)/std1 .* (area_interpimgrid2(:)-mean2)/std2);
            if (curNCC>=maxNCC) 
                maxNCC=curNCC;
                maxk=k/p;
                maxl=l/p;
            end
        end
    end
    k=maxk;
    l=maxl;
end

if (method==1) strmethod='(handpicked points)'; else strmethod=['(NCC = ',num2str(maxNCC),')']; end

disp(['Identified 2pi multiple for registration ',strmethod,':']);            
disp(['x-comp: ',num2str(l),'  y-comp: ',num2str(k)]);

end

