clear all, close all;

% The path must be correctly set, see the documentation.
% For instance, if Matlab's current directory is the root directory of the toolbox (gridmethod_v10), simply type:
% >> addpath data functions scripts


% load images and set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% choose an experiment (values: 1, 2, 3, 4, 5)
experiment = 1;

% choose a registering method (values: 1 for hand-picked corresponding points, 2 for image correlation)
method_register = 2;
% if experiment==5 (synthetic data), use method_register=1

% choose a procedure for displacement, strain, and rotation maps estimation (values: 1 or 2)
procedure = 2;

disp(['experiment = ',num2str(experiment),',  method_register = ',num2str(method_register), ',  procedure = ',num2str(procedure)]);

switch experiment
    
    case 1
    % Experiment 1:
    %%%%%%%%%%%%%%%

    % reference grid image
    photo1=imread('F=0_0001.tif');

    % after deformation test
    photo2=imread('F=5000_0001.tif');

    p=5; % pattern pitch
    strainxx_range=[-0.0012,0.0012]; % for display
    strainyy_range=[-0.0012,0.0012];
    strainxy_range=[-0.0012,0.0012];
    rot_range=[-0.002,0.002];
    ux_range=[-0.1,1.1];
    uy_range=[-17,-15];
    window=[50,1000,150,1000]; % for registration (see below)
    imgrid1=double(photo1(1:1040,110:1290));
    imgrid2=double(photo2(1:1040,110:1290));

    case 2
    % Experiment 2:
    %%%%%%%%%%%%%%%

    % reference grid image
    photo1=imread('F=0_0001.tif');

    % after deformation test
    photo2=imread('F=8100_0001.tif');
    
    p=5; % pattern pitch
    strainxx_range=[-0.02,0.02]; % for display
    strainyy_range=[-0.03,0.03]; 
    strainxy_range=[-0.005,0.005]; 
    rot_range=[-0.02,0.02]; 
    ux_range=[-5,0.8];
    uy_range=[-33,-24];
    window=[50,1000,150,1000]; % for registration (see below)
    imgrid1=double(photo1(1:1040,110:1290));
    imgrid2=double(photo2(1:1040,110:1290));
   
    case 3
    % Experiment 3:
    %%%%%%%%%%%%%%%
 
    % reference grid image
    photo1=imread('flexion_0001.tif');

    % after deformation test
    photo2=imread('flexion_0450.tif');

    p=5; % pattern pitch
    strainxx_range=[-0.006,0.006]; % for display
    strainyy_range=[-0.01,0.006]; 
    strainxy_range=[-0.01,0.006]; 
    rot_range=[-0.02,0];      
    ux_range=[-2,4];
    uy_range=[4,15];
    window=[50,1000,100,1200];
    imgrid1=double(photo1);
    imgrid2=double(photo2);
    
    case 4
    % Experiment 4:
    %%%%%%%%%%%%%%%
 
    % reference grid image
    photo1=imread('compression_0001.tif');

    % after deformation test
    photo2=imread('compression_0050.tif');

    p=5; % pattern pitch
    strainxx_range=[-0.02,0.02]; % for display
    strainyy_range=[-0.02,0.02]; 
    strainxy_range=[-0.015,0.015]; 
    rot_range=[-0.015,0.015]; 
    ux_range=[-1,2];
    uy_range=[2,6];
    window=[100,600,100,1100];
    imgrid1=double(photo1(150:850,100:1300));
    imgrid2=double(photo2(150:850,100:1300));
    
    case 5
    % Experiment 5:
    %%%%%%%%%%%%%%%
 
    % reference grid image
    photo1=imread('ref_synthetic_grid.tif');

    % after deformation test
    photo2=imread('modulated_synthetic_grid.tif');

    % the following values have to match the values in script_generate_synthetic_grids.m
    p=7; % pattern pitch
    strainxx_range=[-0.005,0.005]; % for display
    strainyy_range=[-0.005,0.005]; 
    strainxy_range=[-0.005,0.005]; 
    rot_range=[-0.005,0.005]; 
    ux_range=[-0.2,0];
    uy_range=[-0.05,0.05];
    window=[100,400,100,1900];
    imgrid1=double(photo1);
    imgrid2=double(photo2);
    otherwise
        disp('unknown experiment');
end


% phases and phase derivatives calculation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Gaussian analysis window
profile=0;
sigma=p;
g = build_window( profile, sigma );
% cf documentation for the relation between sigma and profile

% LSA 
[ PHI1_X, PHI1_Y, mod1X, mod1Y ] = LSA( imgrid1, g, p);
[ PHI2_X, PHI2_Y, mod2X, mod2Y ] = LSA( imgrid2, g, p);

% phase unwrapping
PHI1_X=unwrap2D(single(PHI1_X));
PHI1_Y=unwrap2D(single(PHI1_Y));
PHI2_X=unwrap2D(single(PHI2_X));
PHI2_Y=unwrap2D(single(PHI2_Y));


% register gridimage2 to gridimage1
% (up to this point, phases are retrieved up to a multiple of 2pi)

% with hand-picked points, uncomment the two following lines and note the
% coordinates of two corresponding points
% figure, imagesc(imgrid1), colormap(gray);
% figure, imagesc(imgrid2), colormap(gray);
    
switch method_register
    
    case 1  % with hand-picked points
    switch experiment
        case 1
        % for instance, in experiment 1, 
        % pixel (292,154) in image 2 corresponds to pixel (291,170) in image 1
        match=[291,170,292,154];
        case 2
        % for instance, in experiment 2, 
        % pixel (298,137) in image 2 corresponds to pixel (301,170) in image 1
        match=[301,170,298,137];
        case 3
        % for instance, in experiment 3, 
        % pixel (452,450) in image 2 corresponds to pixel (451,441) in image 1
        match=[451,441,452,450];
        case 4
        % for instance, in experiment 4, 
        % pixel (596,170) in image 2 corresponds to pixel (596,164) in image 1
        match=[596,164,596,170]; 
        case 5
        % in experiment 5, automatic registration does not properly work because of the
        % absence of grid defects. 
        % However, the synthetic grid image is deformed "in place".
        match=[500,500,500,500];
    end
    [k,l] = register_gridimages( method_register, match, p, imgrid1, imgrid2, PHI1_X, PHI1_Y, PHI2_X, PHI2_Y );

    case 2  % with image correlation:
    if (experiment~=5)
        [k,l] = register_gridimages( method_register, window, p, imgrid1, imgrid2, PHI1_X, PHI1_Y, PHI2_X, PHI2_Y );
    else
        k=0; l=0;
    end
    
end

PHI2_X=PHI2_X+l*2*pi;
PHI2_Y=PHI2_Y+k*2*pi;



% displacement and strain fields calculation and display
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


switch procedure
    
    case 1
        
        [ UX0, UY0, EPSXX0, EPSYY0, EPSXY0, WXY0 ] = calculate_U_EPS( p, PHI1_X, PHI1_Y, PHI2_X, PHI2_Y, procedure);

        % display
        figure, imagesc(UX0,ux_range), colorbar
        title('UX, LSA, proc 1');
        figure, imagesc(UY0,uy_range), colorbar
        title('UY, LSA, proc 1');
        figure, imagesc(EPSXX0,strainxx_range), colorbar
        title('EPSX, LSA, proc 1');
        figure, imagesc(EPSYY0,strainyy_range), colorbar
        title('EPSY, LSA, proc 1');
        figure, imagesc(EPSXY0,strainxy_range), colorbar
        title('EPSXY, LSA, proc 1');
        figure, imagesc(WXY0,rot_range), colorbar
        title('WXY, LSA, proc 1');
        % Note spurious fringes caused by small variations in the grid pitch
        % due to manufacturing constraints and localized spots caused by lack of
        % ink in the grid

    case 2

        [ UX1, UY1, EPSXX1, EPSYY1, EPSXY1, WXY1 ] = calculate_U_EPS( p, PHI1_X, PHI1_Y, PHI2_X, PHI2_Y, procedure);
        % for an iterative estimation (with maxiter=50), use:
        % [ UX1, UY1, EPSXX1, EPSYY1, EPSXY1, WXY1 ] = calculate_U_EPS( p, PHI1_X, PHI1_Y, PHI2_X, PHI2_Y, procedure, 50);

        % display
        figure, imagesc(UX1,ux_range), colorbar
        title('UX, LSA, proc 2');
        figure, imagesc(UY1,uy_range), colorbar
        title('UY, LSA, proc 2');
        figure, imagesc(EPSXX1,strainxx_range), colorbar
        title('EPSX, LSA, proc 2');
        figure, imagesc(EPSYY1,strainyy_range), colorbar
        title('EPSY, LSA, proc 2');
        figure, imagesc(EPSXY1,strainxy_range), colorbar
        title('EPSXY, LSA, proc 2');
        figure, imagesc(WXY1,rot_range), colorbar
        title('WXY, LSA, proc 2');
        
end;
